#include "common.hh"
#include "cylinder.hh"
#include "params.hh"
#include <cmath>

Cylinder::Cylinder (const Vector& begin, const Vector& end, double rad):
  start(begin), dir((end-begin).normalize()), r(rad), r2(rad*rad), 
  l((end-begin).length()) {}

const void Cylinder::inside (Vector& point, const Vector& old) const {
  Vector diff1 = point-start;
  double d1 = diff1.dot (dir);
  if ((d1 > 0) && (d1 < l)) {
    Vector tmp1 = diff1 - d1*dir;
    if (tmp1.length() < r) {
      Vector diff2 = old-start;
      double d_2 = diff2.dot (dir);
      Vector tmp2 = diff2 - d_2*dir;
      double lambda;
      if (d_2 > 0) {
        if(d_2 < l) {
          lambda = (r-tmp2.length())/(EPSILON+tmp1.length()-tmp2.length());
        } else {
          lambda = (l-d_2)/(-EPSILON+d1-d_2);
        }
      } else {
        lambda = -d_2/(-EPSILON+d1-d_2);
      }
      point = old+lambda*(point-old);
    }
  }
}

const void Cylinder::adjust (Vector& force, const Vector& point) const {
  Vector diff = point - start;
  double d1 = diff.dot (dir);
  if (d1 <= -EPSILON) return;
  if (d1 >= l+EPSILON) return;
  Vector u = (diff - d1*dir);
  double r1 = u.length();
  if (r1 >= r+EPSILON) return;
  if (d1 >= l) {                // on the top plane
    double f = force.dot (dir);
    Vector tmp = force - f*dir;
    if (f < 0)
      force = tmp;
    if (tmp.length() < (friction + EPSILON))
      force -= tmp;
    else
      force -= (friction/tmp.length())*tmp;
  } else if (d1 <= 0) {         // on the bottom plane
    double f = force.dot (dir);
    Vector tmp = force - f*dir;
    if (f > 0)
      force = tmp;
    if (tmp.length() < (friction + EPSILON))
      force -= tmp;
    else
      force -= (friction/tmp.length())*tmp;
  } else {                      // neither, therefore on the side
    double f = force.dot (u);
    Vector tmp = force - (f/r1)*u;
    if (f < 0)
      force = tmp;
    if (tmp.length() < (friction + EPSILON))
      force -= tmp;
    else
      force -= (friction/tmp.length())*tmp;
  }
}

