#include "common.hh"
#include "vector.hh"
#include <cstring>
#include <cmath>

const Vector O (0, 0, 0);
const Vector X (1, 0, 0);
const Vector Y (0, 1, 0);
const Vector Z (0, 0, 1);

static class VectorInit {
public:
  VectorInit() {
    O.length(); X.length(); Y.length(); Z.length();
  }
} initializer;

Vector::Vector (double x, double y, double z): coords (), l_(-1.0) {
  coords[0] = x;
  coords[1] = y;
  coords[2] = z;
}

Vector::Vector (const Vector& v): coords(), l_() {
  memcpy (coords, v.coords, sizeof (coords));
  l_ = v.l_;
}

const Vector Vector::operator+ (const Vector& v) const {
  return Vector (coords[0]+v.coords[0], coords[1]+v.coords[1], coords[2]+v.coords[2]);
}
const Vector Vector::operator+() const {
  return *this;
}
const Vector Vector::operator- (const Vector& v) const {
  return Vector (coords[0]-v.coords[0], coords[1]-v.coords[1], coords[2]-v.coords[2]);
}
const Vector Vector::operator-() const {
  return Vector (-coords[0], -coords[1], -coords[2]);
}

Vector& Vector::operator+= (const Vector& v) {
  coords[0] += v.coords[0];
  coords[1] += v.coords[1];
  coords[2] += v.coords[2];
  l_ = -1;
  return *this;
}
Vector& Vector::operator-= (const Vector& v) {
  coords[0] -= v.coords[0];
  coords[1] -= v.coords[1];
  coords[2] -= v.coords[2];
  l_ = -1;
  return *this;
}
Vector& Vector::operator*= (const double l) {
  coords[0] *= l;
  coords[1] *= l;
  coords[2] *= l;
  l_ = -1;
  return *this;
}

const Vector operator* (const double l, const Vector& v) {
  return Vector (l*v.coords[0], l*v.coords[1], l*v.coords[2]);
}

ostream& operator<< (ostream& o, const Vector& v) {
  o << "<" << v.x() << ", " << v.z() << ", " << v.y() << ">";
  return o;
}

const bool Vector::operator== (const Vector& v) const {
  return ((*this)-v).length() < EPSILON;
}

const double Vector::length() const {
  if (l_ < 0) {
    double tmp = coords[0]*coords[0] + coords[1]*coords[1] + coords[2]*coords[2];
    if (!isfinite(tmp)) {
      cerr << tmp << " is not finite!" << endl;
      exit(1);
    }
    l_ = (tmp>EPSILON)?sqrt (tmp):0;
  }
  return l_;
}
const Vector Vector::normalize() const {
  double l = 1/length();
  return l*(*this);
}
const double Vector::dot (const Vector& v) const {
  return coords[0]*v.coords[0] + coords[1]*v.coords[1] + coords[2]*v.coords[2];
}
const Vector Vector::cross (const Vector& v) const {
  return Vector (coords[1]*v.coords[2]-coords[2]*v.coords[1],
                 coords[2]*v.coords[0]-coords[0]*v.coords[2],
                 coords[0]*v.coords[1]-coords[1]*v.coords[0]);
}
